Imports System
Imports System.Collections.Generic
Imports System.Text

Namespace Helpers

    NotInheritable Class FileHelper
        Public Interface IFormatSet
        End Interface

        Public Class FormatSet

            Public MinPrefix As DecimalPrefix
            Public MaxPrefix As DecimalPrefix
            Public Standard As PrefixStandard
            Public Decimals As UShort
            Public UseThousandsSeparator As Boolean

            Public Sub New()
                MinPrefix = DecimalPrefix.None
                MaxPrefix = DecimalPrefix.Exa
                Standard = PrefixStandard.Binary
                Decimals = 2
                UseThousandsSeparator = True
            End Sub
        End Class

        Private Structure DecimalFileSize
            Private FileSize As Long
            Private PrefixedFileSize As Double
            Private Prefix As DecimalPrefix
        End Structure

        Private Structure BinaryFileSize
            Private FileSize As Long
            Private PrefixedFileSize As Double
            Private Prefix As BinaryPrefix
        End Structure

        Public Enum PrefixStandard
            [Decimal] = 0
            Binary = 1
        End Enum

        Public Enum FormatSets
            DefaultWindowsFileSize
            DefaultInternetSpeed
        End Enum

        Public Class FileSizeSuffix
#Region "Decimal (SI) suffixes"
            Public Const [Byte] As String = "B"
            Public Const KiloByte As String = "kB"
            Public Const MegaByte As String = "MB"
            Public Const GigaByte As String = "GB"
            Public Const TeraByte As String = "TB"
            Public Const PetaByte As String = "PB"
            Public Const ExaByte As String = "EB"
            Public Const ZettaByte As String = "ZB"
            Public Const YottaByte As String = "YB"
#End Region

#Region "Binary suffixes"
            Public Const KibiByte As String = "KiB"
            Public Const MebiByte As String = "MiB"
            Public Const GibiByte As String = "GiB"
            Public Const TebiByte As String = "TiB"
            Public Const PebiByte As String = "PiB"
            Public Const ExbiByte As String = "EiB"
            Public Const ZebiByte As String = "ZiB"
            Public Const YobiByte As String = "YiB"
#End Region
        End Class

        Public Class FileSizeName
#Region "Decimal (SI) names"
            Public Const KiloByte As String = "kilobyte"
#End Region

#Region "Binary names"
            Public Const KibiByte As String = "kibibyte"
            Public Const MebiByte As String = "mebibyte"
#End Region
        End Class

        Public Enum SizeResolution
            [Byte]
            Bit
        End Enum

#Region "Prefix methods"

        Private Function GetPreferredDecimalPrefix(ByVal fileSize As Long) As DecimalPrefix
            Return GetPreferredDecimalPrefix(fileSize, DecimalPrefix.None, DecimalPrefix.Exa)
        End Function

        Private Function GetPreferredDecimalPrefix(ByVal fileSize As Long, ByVal minPrefix As DecimalPrefix, ByVal maxPrefix As DecimalPrefix) As DecimalPrefix
            If checkPrefix(fileSize, DecimalPrefix.Exa, minPrefix, maxPrefix) Then
                Return DecimalPrefix.Exa
            ElseIf checkPrefix(fileSize, DecimalPrefix.Peta, minPrefix, maxPrefix) Then
                Return DecimalPrefix.Peta
            ElseIf checkPrefix(fileSize, DecimalPrefix.Tera, minPrefix, maxPrefix) Then
                Return DecimalPrefix.Tera
            ElseIf checkPrefix(fileSize, DecimalPrefix.Giga, minPrefix, maxPrefix) Then
                Return DecimalPrefix.Giga
            ElseIf checkPrefix(fileSize, DecimalPrefix.Mega, minPrefix, maxPrefix) Then
                Return DecimalPrefix.Mega
            ElseIf checkPrefix(fileSize, DecimalPrefix.Kilo, minPrefix, maxPrefix) Then
                Return DecimalPrefix.Kilo
            Else
                Return DecimalPrefix.None
            End If
        End Function

        Private Function GetPreferredBinaryPrefix(ByVal fileSize As Long) As BinaryPrefix
            Return GetPreferredBinaryPrefix(fileSize, BinaryPrefix.None, BinaryPrefix.Exbi)
        End Function

        Private Function GetPreferredBinaryPrefix(ByVal fileSize As Long, ByVal minPrefix As BinaryPrefix, ByVal maxPrefix As BinaryPrefix) As BinaryPrefix
            If checkPrefix(fileSize, BinaryPrefix.Exbi, minPrefix, maxPrefix) Then
                Return BinaryPrefix.Exbi
            ElseIf checkPrefix(fileSize, BinaryPrefix.Pebi, minPrefix, maxPrefix) Then
                Return BinaryPrefix.Pebi
            ElseIf checkPrefix(fileSize, BinaryPrefix.Tebi, minPrefix, maxPrefix) Then
                Return BinaryPrefix.Tebi
            ElseIf checkPrefix(fileSize, BinaryPrefix.Gibi, minPrefix, maxPrefix) Then
                Return BinaryPrefix.Gibi
            ElseIf checkPrefix(fileSize, BinaryPrefix.Mebi, minPrefix, maxPrefix) Then
                Return BinaryPrefix.Mebi
            ElseIf checkPrefix(fileSize, BinaryPrefix.Kibi, minPrefix, maxPrefix) Then
                Return BinaryPrefix.Kibi
            Else
                Return BinaryPrefix.None
            End If
        End Function

        Private Function checkPrefix(ByVal fileSize As Long, ByVal prefix As DecimalPrefix, ByVal minPrefix As DecimalPrefix, ByVal maxPrefix As DecimalPrefix) As Boolean
            If (prefix < minPrefix) OrElse (prefix > maxPrefix) Then
                Return False
            End If

            Return (fileSize >= CLng(prefix))
        End Function

        Private Function checkPrefix(ByVal fileSize As Long, ByVal prefix As BinaryPrefix, ByVal minPrefix As BinaryPrefix, ByVal maxPrefix As BinaryPrefix) As Boolean
            If (prefix < minPrefix) OrElse (prefix > maxPrefix) Then
                Return False
            End If

            Return (fileSize >= CLng(prefix))
        End Function
#End Region


        Public Enum DecimalPrefix As Long
            None = 0
            Kilo = 1000
            ' Pow(10, 3) 
            Mega = 1000000
            ' Pow(10, 6) 
            Giga = 1000000000
            ' Pow(10, 9) 
            Tera = 1000000000000
            ' Pow(10, 12) 
            Peta = 1000000000000000
            ' Pow(10, 15) 
            Exa = 1000000000000000000
            ' Pow(10, 18) 
        End Enum

        Public Enum BinaryPrefix As Long
            None = 0
            Kibi = 1024
            ' Pow(2, 10) 
            Mebi = 1048576
            ' Pow(2, 20) 
            Gibi = 1073741824
            ' Pow(2, 30) 
            Tebi = 1099511627776
            ' Pow(2, 40) 
            Pebi = 1125899906842624
            ' Pow(2, 50) 
            Exbi = 1152921504606846976
            ' Pow(2, 60) 
        End Enum

        'public class FileSize 
        '{ 
        ' #region Decimal (SI) sizes 
        ' public const long KiloByte = 1000; // Pow(10, 3) 
        ' public const long MegaByte = 1000000; // Pow(10, 6) 
        ' public const long GigaByte = 1000000000; // Pow(10, 9) 
        ' public const long TeraByte = 1000000000000; // Pow(10, 12) 
        ' public const long PetaByte = 1000000000000000; // Pow(10, 15) 
        ' public const long ExaByte = 1000000000000000000; // Pow(10, 18) 
        ' // Unsupported sizes 
        ' //public const int ZettaByte = 1000000000000000000000; // Pow(10, 21) 
        ' //public const int YottaByte = 1000000000000000000000000; // Pow(10, 24) 
        ' #endregion 

        ' #region Binary sizes 
        ' public const long KibiByte = 1024; // Pow(2, 10) 
        ' public const long MebiByte = 1048576; // Pow(2, 20) 
        ' public const long GibiByte = 1073741824; // Pow(2, 30) 
        ' public const long TebiByte = 1099511627776; // Pow(2, 40) 
        ' public const long PebiByte = 1125899906842624; // Pow(2, 50) 
        ' public const long ExbiByte = 1152921504606846976; // Pow(2, 60) 
        ' // Unsupported sizes 
        ' //public const ? ZebiByte = 1180591620717411303424; // Pow(2, 70) 
        ' //public const ? YobiByte = 1208925819614629174706176; // Pow(2, 80) 
        ' #endregion 

        '} 

        ''' <summary>Returns a file size as a formatted string with a unit.</summary> 
        ''' <param name="fileSize">The file size to format.</param> 
        ''' <returns>The formatted string.</returns> 
        Public Shared Function FormatFileSize(ByVal fileSize As Long) As String
            If fileSize < 0 Then
                Throw New ArgumentOutOfRangeException("fileSize")
            End If

            Return FormatFileSize(CType(fileSize, ULong))
        End Function

        Public Shared Function Convert(ByVal value As Long) As Long
            Return 0
        End Function

        ''' <summary>Returns a file size as a formatted string with a unit.</summary> 
        ''' <param name="fileSize">The file size to format.</param> 
        ''' <returns>The formatted string.</returns> 
        Public Shared Function FormatFileSize(ByVal fileSize As ULong) As String


            If fileSize >= 1024 * 1024 * 1024 Then
                Return String.Format("{0:########0.00} GB", CDbl(fileSize) / (1024 * 1024 * 1024))
            ElseIf fileSize >= 1024 * 1024 Then
                Return String.Format("{0:####0.00} MB", CDbl(fileSize) / (1024 * 1024))
            ElseIf fileSize >= 1024 Then
                Return String.Format("{0:####0.00} KB", CDbl(fileSize) / 1024)
            Else
                Return String.Format("{0} bytes", fileSize)
            End If
        End Function

        ''' <summary>Returns a file size as a formatted string with a unit.</summary> 
        ''' <param name="fileSize">The file size to format.</param> 
        ''' <returns>The formatted string.</returns> 
        Public Shared Function FormatFileSize(ByVal fileSize As Double) As String


            If fileSize >= 1024 * 1024 * 1024 Then
                Return String.Format("{0:########0.00} GB", CDbl(fileSize) / (1024 * 1024 * 1024))
            ElseIf fileSize >= 1024 * 1024 Then
                Return String.Format("{0:####0.00} MB", CDbl(fileSize) / (1024 * 1024))
            ElseIf fileSize >= 1024 Then
                Return String.Format("{0:####0.00} KB", CDbl(fileSize) / 1024)
            Else
                Return String.Format("{0} bytes", fileSize)
            End If
        End Function

    End Class
End Namespace